@@ -30,15 +30,26 @@
 
 #include "ak.h"
 #include "akaudiocaps.h"
+#include "akaudioconverter.h"
 #include "akaudiopacket.h"
 #include "akcaps.h"
+#include "akcolorcomponent.h"
+#include "akcolorconvert.h"
+#include "akcolorplane.h"
+#include "akcompressedvideocaps.h"
+#include "akcompressedvideopacket.h"
 #include "akelement.h"
 #include "akfrac.h"
 #include "akpacket.h"
 #include "akplugininfo.h"
 #include "akpluginmanager.h"
+#include "aksubtitlecaps.h"
+#include "aksubtitlepacket.h"
 #include "akunit.h"
 #include "akvideocaps.h"
+#include "akvideoconverter.h"
+#include "akvideoformatspec.h"
+#include "akvideomixer.h"
 #include "akvideopacket.h"
 #include "qml/akcolorizedimage.h"
 #include "qml/akutils.h"
@@ -85,21 +96,32 @@ void Ak::registerTypes()
         return new Ak();
     });
     AkAudioCaps::registerTypes();
+    AkAudioConverter::registerTypes();
     AkAudioPacket::registerTypes();
     AkCaps::registerTypes();
+    AkColorComponent::registerTypes();
+    AkColorConvert::registerTypes();
+    AkColorPlane::registerTypes();
+    AkColorizedImage::registerTypes();
+    AkCompressedVideoCaps::registerTypes();
+    AkCompressedVideoPacket::registerTypes();
     AkElement::registerTypes();
     AkFrac::registerTypes();
     AkPacket::registerTypes();
+    AkPalette::registerTypes();
+    AkPaletteGroup::registerTypes();
     AkPluginInfo::registerTypes();
     AkPluginManager::registerTypes();
-    AkUnit::registerTypes();
-    AkVideoCaps::registerTypes();
-    AkVideoPacket::registerTypes();
-    AkColorizedImage::registerTypes();
+    AkSubtitleCaps::registerTypes();
+    AkSubtitlePacket::registerTypes();
     AkTheme::registerTypes();
-    AkPaletteGroup::registerTypes();
-    AkPalette::registerTypes();
+    AkUnit::registerTypes();
     AkUtils::registerTypes();
+    AkVideoCaps::registerTypes();
+    AkVideoConverter::registerTypes();
+    AkVideoFormatSpec::registerTypes();
+    AkVideoMixer::registerTypes();
+    AkVideoPacket::registerTypes();
 }
 
 qint64 Ak::id()
@@ -109,6 +131,21 @@ qint64 Ak::id()
     return id++;
 }
 
+QString Ak::platform()
+{
+#ifdef Q_OS_WIN32
+    return {"windows"};
+#elif defined(Q_OS_OSX)
+    return {"macos"};
+#elif defined(Q_OS_ANDROID)
+    return {"android"};
+#elif defined(Q_OS_POSIX)
+    return {"posix"};
+#else
+    return {"unknown"};
+#endif
+}
+
 void Ak::setQmlEngine(QQmlEngine *engine)
 {
     if (engine == akGlobalStuff->m_globalEngine)
