@@ -36,10 +36,6 @@
 #define PAUSE_TIMEOUT 500
 #define DUMMY_OUTPUT_DEVICE ":dummyout:"
 
-#ifdef Q_OS_WIN32
-#include <combaseapi.h>
-#endif
-
 using AudioDevPtr = QSharedPointer<AudioDev>;
 
 class AudioDeviceElementPrivate
@@ -121,12 +117,12 @@ QString AudioDeviceElement::defaultInput()
     QString defaultInput;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        defaultInput = this->d->m_audioDevice->defaultInput();
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        defaultInput = audioDevice->defaultInput();
+
     return defaultInput;
 }
 
@@ -135,12 +131,12 @@ QString AudioDeviceElement::defaultOutput()
     QString defaultOutput;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        defaultOutput = this->d->m_audioDevice->defaultOutput();
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        defaultOutput = audioDevice->defaultOutput();
+
     return defaultOutput;
 }
 
@@ -162,12 +158,12 @@ QString AudioDeviceElement::description(const QString &device)
     QString description;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        description = this->d->m_audioDevice->description(device);
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        description = audioDevice->description(device);
+
     return description;
 }
 
@@ -178,8 +174,12 @@ QString AudioDeviceElement::device() const
 
 int AudioDeviceElement::latency() const
 {
-    if (this->d->m_audioDevice)
-        return  this->d->m_audioDevice->latency();
+    this->d->m_mutexLib.lock();
+    auto audioDevice = this->d->m_audioDevice;
+    this->d->m_mutexLib.unlock();
+
+    if (audioDevice)
+        return audioDevice->latency();
 
     return 25;
 }
@@ -194,17 +194,18 @@ AkAudioCaps AudioDeviceElement::preferredFormat(const QString &device)
     if (device == DUMMY_OUTPUT_DEVICE)
         return AkAudioCaps(AkAudioCaps::SampleFormat_s16,
                            AkAudioCaps::Layout_stereo,
+                           false,
                            44100);
 
     AkAudioCaps preferredFormat;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        preferredFormat = this->d->m_audioDevice->preferredFormat(device);
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        preferredFormat = audioDevice->preferredFormat(device);
+
     return preferredFormat;
 }
 
@@ -221,12 +222,12 @@ QList<AkAudioCaps::SampleFormat> AudioDeviceElement::supportedFormats(const QStr
     QList<AkAudioCaps::SampleFormat> supportedFormats;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        supportedFormats = this->d->m_audioDevice->supportedFormats(device);
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        supportedFormats = audioDevice->supportedFormats(device);
+
     return supportedFormats;
 }
 
@@ -241,13 +242,12 @@ QList<AkAudioCaps::ChannelLayout> AudioDeviceElement::supportedChannelLayouts(co
     QList<AkAudioCaps::ChannelLayout> supportedChannelLayouts;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        supportedChannelLayouts =
-                this->d->m_audioDevice->supportedChannelLayouts(device);
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        supportedChannelLayouts = audioDevice->supportedChannelLayouts(device);
+
     return supportedChannelLayouts;
 }
 
@@ -259,12 +259,12 @@ QList<int> AudioDeviceElement::supportedSampleRates(const QString &device)
     QList<int> supportedSampleRates;
 
     this->d->m_mutexLib.lock();
-
-    if (this->d->m_audioDevice)
-        supportedSampleRates = this->d->m_audioDevice->supportedSampleRates(device);
-
+    auto audioDevice = this->d->m_audioDevice;
     this->d->m_mutexLib.unlock();
 
+    if (audioDevice)
+        supportedSampleRates = audioDevice->supportedSampleRates(device);
+
     return supportedSampleRates;
 }
 
@@ -283,20 +283,19 @@ AudioDeviceElementPrivate::AudioDeviceElementPrivate(AudioDeviceElement *self):
 
 void AudioDeviceElementPrivate::readFramesLoop()
 {
-    if (!this->m_audioDevice)
-        return;
+    this->m_mutexLib.lock();
+    auto audioDevice = this->m_audioDevice;
+    this->m_mutexLib.unlock();
 
-#ifdef Q_OS_WIN32
-    // Initialize the COM library in multithread mode.
-    CoInitializeEx(nullptr, COINIT_MULTITHREADED);
-#endif
+    if (!audioDevice)
+        return;
 
     QString device = this->m_device;
     AkAudioCaps caps(this->m_caps);
     qint64 streamId = Ak::id();
     AkFrac timeBase(1, caps.rate());
 
-    if (this->m_audioDevice->init(device, caps)) {
+    if (audioDevice->init(device, caps)) {
         while (this->m_readFramesLoop) {
             if (this->m_pause) {
                 QThread::msleep(PAUSE_TIMEOUT);
@@ -304,19 +303,18 @@ void AudioDeviceElementPrivate::readFramesLoop()
                 continue;
             }
 
-            auto buffer = this->m_audioDevice->read();
+            auto buffer = audioDevice->read();
 
             if (buffer.isEmpty())
                 return;
 
-            QByteArray oBuffer(buffer.size(), 0);
-            memcpy(oBuffer.data(), buffer.constData(), size_t(buffer.size()));
+            size_t samples = 8 * buffer.size() / (caps.channels() * caps.bps());
+            AkAudioPacket packet(caps, samples);
+            memcpy(packet.data(),
+                   buffer.constData(),
+                   qMin<size_t>(packet.size(), buffer.size()));
             auto pts = qint64(QTime::currentTime().msecsSinceStartOfDay()
                               / timeBase.value() / 1e3);
-            caps.setSamples(8 * buffer.size() / (caps.channels() * caps.bps()));
-            AkAudioPacket packet;
-            packet.caps() = caps;
-            packet.buffer() = oBuffer;
             packet.setPts(pts);
             packet.setTimeBase(timeBase);
             packet.setIndex(0);
@@ -325,13 +323,8 @@ void AudioDeviceElementPrivate::readFramesLoop()
             emit self->oStream(packet);
         }
 
-        this->m_audioDevice->uninit();
+        audioDevice->uninit();
     }
-
-#ifdef Q_OS_WIN32
-    // Close COM library.
-    CoUninitialize();
-#endif
 }
 
 void AudioDeviceElementPrivate::setInputs(const QStringList &inputs)
@@ -364,11 +357,15 @@ void AudioDeviceElementPrivate::linksChanged(const AkPluginLinks &links)
     bool isInput = this->m_inputs.contains(this->m_device);
 
     this->m_mutexLib.lock();
+    auto audioDevice = this->m_audioDevice;
+    this->m_mutexLib.unlock();
+
     int latency = 25;
 
-    if (this->m_audioDevice)
-        latency = this->m_audioDevice->latency();
+    if (audioDevice)
+        latency = audioDevice->latency();
 
+    this->m_mutexLib.lock();
     this->m_audioDevice =
             akPluginManager->create<AudioDev>("AudioSource/AudioDevice/Impl/*");
     this->m_mutexLib.unlock();
@@ -423,35 +420,40 @@ void AudioDeviceElementPrivate::linksChanged(const AkPluginLinks &links)
 
 AkPacket AudioDeviceElement::iAudioStream(const AkAudioPacket &packet)
 {
-    if (!this->d->m_audioDevice)
-        return AkPacket();
+    if (!packet)
+        return {};
+
+    this->d->m_mutexLib.lock();
+    auto audioDevice = this->d->m_audioDevice;
+    this->d->m_mutexLib.unlock();
+
+    if (!audioDevice)
+        return {};
 
     this->d->m_mutex.lock();
 
     if (this->state() != ElementStatePlaying) {
         this->d->m_mutex.unlock();
 
-        return AkPacket();
+        return {};
     }
 
     auto device = this->d->m_device;
     this->d->m_mutex.unlock();
 
-    if (device == DUMMY_OUTPUT_DEVICE)
+    if (device == DUMMY_OUTPUT_DEVICE) {
+#ifndef Q_OS_ANDROID
         QThread::usleep(ulong(1e6
-                              * packet.caps().samples()
+                              * packet.samples()
                               / packet.caps().rate()));
-    else {
-        AkPacket iPacket;
+#endif
+    } else {
         this->d->m_mutex.lock();
-        iPacket = this->d->m_audioConvert.convert(packet);
+        auto iPacket = this->d->m_audioConvert.convert(packet);
         this->d->m_mutex.unlock();
 
-        if (iPacket) {
-            this->d->m_mutexLib.lock();
-            this->d->m_audioDevice->write(iPacket);
-            this->d->m_mutexLib.unlock();
-        }
+        if (iPacket)
+            audioDevice->write(iPacket);
     }
 
     return AkPacket();
@@ -464,18 +466,27 @@ void AudioDeviceElement::setDevice(const QString &device)
 
     this->d->m_device = device;
     emit this->deviceChanged(device);
+
+    this->d->m_mutexLib.lock();
+    auto audioDevice = this->d->m_audioDevice;
+    this->d->m_mutexLib.unlock();
+
     AkAudioCaps preferredFormat;
 
-    if (this->d->m_audioDevice)
-        preferredFormat = this->d->m_audioDevice->preferredFormat(device);
+    if (audioDevice)
+        preferredFormat = audioDevice->preferredFormat(device);
 
     this->setCaps(preferredFormat);
 }
 
 void AudioDeviceElement::setLatency(int latency)
 {
-    if (this->d->m_audioDevice)
-        this->d->m_audioDevice->setLatency(latency);
+    this->d->m_mutexLib.lock();
+    auto audioDevice = this->d->m_audioDevice;
+    this->d->m_mutexLib.unlock();
+
+    if (audioDevice)
+        audioDevice->setLatency(latency);
 }
 
 void AudioDeviceElement::setCaps(const AkAudioCaps &caps)
@@ -495,22 +506,27 @@ void AudioDeviceElement::resetDevice()
 
 void AudioDeviceElement::resetLatency()
 {
-    if (this->d->m_audioDevice)
-        this->d->m_audioDevice->resetLatency();
+    this->d->m_mutexLib.lock();
+    auto audioDevice = this->d->m_audioDevice;
+    this->d->m_mutexLib.unlock();
+
+    if (audioDevice)
+        audioDevice->resetLatency();
 }
 
 void AudioDeviceElement::resetCaps()
 {
-    this->d->m_mutexLib.lock();
     auto preferredFormat = this->preferredFormat(this->d->m_device);
-    this->d->m_mutexLib.unlock();
-
     this->setCaps(preferredFormat);
 }
 
 bool AudioDeviceElement::setState(AkElement::ElementState state)
 {
-    if (!this->d->m_audioDevice)
+    this->d->m_mutexLib.lock();
+    auto audioDevice = this->d->m_audioDevice;
+    this->d->m_mutexLib.unlock();
+
+    if (!audioDevice)
         return false;
 
     AkElement::ElementState curState = this->state();
@@ -545,10 +561,7 @@ bool AudioDeviceElement::setState(AkElement::ElementState state)
                 this->d->m_audioConvert.reset();
                 QString device = this->d->m_device;
                 AkAudioCaps caps(this->d->m_caps);
-
-                this->d->m_mutexLib.lock();
-                auto isInit = this->d->m_audioDevice->init(device, caps);
-                this->d->m_mutexLib.unlock();
+                auto isInit = audioDevice->init(device, caps);
 
                 if (!isInit)
                     return false;
@@ -571,9 +584,7 @@ bool AudioDeviceElement::setState(AkElement::ElementState state)
                 this->d->m_readFramesLoopResult.waitForFinished();
             } else if (this->d->m_device != DUMMY_OUTPUT_DEVICE
                        && this->d->m_outputs.contains(this->d->m_device)) {
-                this->d->m_mutexLib.lock();
-                this->d->m_audioDevice->uninit();
-                this->d->m_mutexLib.unlock();
+                audioDevice->uninit();
             }
 
             return AkElement::setState(state);
@@ -582,12 +593,8 @@ bool AudioDeviceElement::setState(AkElement::ElementState state)
                 this->d->m_pause = false;
             } else if (this->d->m_device != DUMMY_OUTPUT_DEVICE
                        && this->d->m_outputs.contains(this->d->m_device)) {
-                QString device = this->d->m_device;
-                AkAudioCaps caps(this->d->m_caps);
-
-                this->d->m_mutexLib.lock();
-                auto isInit = this->d->m_audioDevice->init(device, caps);
-                this->d->m_mutexLib.unlock();
+                auto isInit = audioDevice->init(this->d->m_device,
+                                                this->d->m_caps);
 
                 if (!isInit)
                     return false;
@@ -609,9 +616,7 @@ bool AudioDeviceElement::setState(AkElement::ElementState state)
                 this->d->m_readFramesLoopResult.waitForFinished();
             } else if (this->d->m_device != DUMMY_OUTPUT_DEVICE
                        && this->d->m_outputs.contains(this->d->m_device)) {
-                this->d->m_mutexLib.lock();
-                this->d->m_audioDevice->uninit();
-                this->d->m_mutexLib.unlock();
+                audioDevice->uninit();
             }
 
             return AkElement::setState(state);
@@ -620,9 +625,7 @@ bool AudioDeviceElement::setState(AkElement::ElementState state)
                 this->d->m_pause = true;
             } else if (this->d->m_device != DUMMY_OUTPUT_DEVICE
                        && this->d->m_outputs.contains(this->d->m_device)) {
-                this->d->m_mutexLib.lock();
-                this->d->m_audioDevice->uninit();
-                this->d->m_mutexLib.unlock();
+                audioDevice->uninit();
             }
 
             return AkElement::setState(state);
