@@ -17,10 +17,16 @@
  * Web-Site: http://webcamoid.github.io/
  */
 
+#include <QDebug>
 #include <QMap>
-#include <QVector>
 #include <QMutex>
+#include <QVector>
 #include <QWaitCondition>
+
+#ifdef Q_OS_ANDROID
+#include <QtAndroid>
+#endif
+
 #include <akaudiopacket.h>
 #include <SLES/OpenSLES_Android.h>
 
@@ -28,6 +34,37 @@
 
 #define N_BUFFERS 4
 
+using SlErrorToStrMap = QMap<SLresult, QString>;
+
+inline const SlErrorToStrMap initSlErrorToStrMap()
+{
+    const SlErrorToStrMap slErrorToStrMap {
+        {SL_RESULT_SUCCESS               , "SL_RESULT_SUCCESS"               },
+        {SL_RESULT_PRECONDITIONS_VIOLATED, "SL_RESULT_PRECONDITIONS_VIOLATED"},
+        {SL_RESULT_PARAMETER_INVALID     , "SL_RESULT_PARAMETER_INVALID"     },
+        {SL_RESULT_MEMORY_FAILURE        , "SL_RESULT_MEMORY_FAILURE"        },
+        {SL_RESULT_RESOURCE_ERROR        , "SL_RESULT_RESOURCE_ERROR"        },
+        {SL_RESULT_RESOURCE_LOST         , "SL_RESULT_RESOURCE_LOST"         },
+        {SL_RESULT_IO_ERROR              , "SL_RESULT_IO_ERROR"              },
+        {SL_RESULT_BUFFER_INSUFFICIENT   , "SL_RESULT_BUFFER_INSUFFICIENT"   },
+        {SL_RESULT_CONTENT_CORRUPTED     , "SL_RESULT_CONTENT_CORRUPTED"     },
+        {SL_RESULT_CONTENT_UNSUPPORTED   , "SL_RESULT_CONTENT_UNSUPPORTED"   },
+        {SL_RESULT_CONTENT_NOT_FOUND     , "SL_RESULT_CONTENT_NOT_FOUND"     },
+        {SL_RESULT_PERMISSION_DENIED     , "SL_RESULT_PERMISSION_DENIED"     },
+        {SL_RESULT_FEATURE_UNSUPPORTED   , "SL_RESULT_FEATURE_UNSUPPORTED"   },
+        {SL_RESULT_INTERNAL_ERROR        , "SL_RESULT_INTERNAL_ERROR"        },
+        {SL_RESULT_UNKNOWN_ERROR         , "SL_RESULT_UNKNOWN_ERROR"         },
+        {SL_RESULT_OPERATION_ABORTED     , "SL_RESULT_OPERATION_ABORTED"     },
+        {SL_RESULT_CONTROL_LOST          , "SL_RESULT_CONTROL_LOST"          },
+    };
+
+    return slErrorToStrMap;
+}
+
+Q_GLOBAL_STATIC_WITH_ARGS(SlErrorToStrMap,
+                          slErrorToStrMap,
+                          (initSlErrorToStrMap()))
+
 class AudioDevOpenSLPrivate
 {
     public:
@@ -63,6 +100,7 @@ class AudioDevOpenSLPrivate
         static SLuint32 channelMaskFromLayout(AkAudioCaps::ChannelLayout layout);
         static void sampleProcess(SLAndroidSimpleBufferQueueItf bufferQueue,
                                   void *context);
+        static bool hasAudioCapturePermissions();
         void updateDevices();
 };
 
@@ -138,34 +176,62 @@ bool AudioDevOpenSL::init(const QString &device, const AkAudioCaps &caps)
 
     this->d->m_samples = qMax(this->latency() * caps.rate() / 1000, 1);
     this->d->m_curCaps = caps;
-    this->d->m_curCaps.setSamples(this->latency() * caps.rate() / 1000);
 
     if (device == ":openslinput:") {
         this->d->m_audioRecorder =
                 AudioDevOpenSLPrivate::createAudioRecorder(this->d->m_engine,
                                                            caps);
 
-        if (!this->d->m_audioRecorder)
-            goto init_failed;
+        if (!this->d->m_audioRecorder) {
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         SLPlayItf audioRecorder = nullptr;
 
         if ((*this->d->m_audioRecorder)->GetInterface(this->d->m_audioRecorder,
                                                       SL_IID_RECORD,
-                                                      &audioRecorder) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                                      &audioRecorder) != SL_RESULT_SUCCESS) {
+            (*this->d->m_audioRecorder)->Destroy(this->d->m_audioRecorder);
+            this->d->m_audioRecorder = nullptr;
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         SLAndroidSimpleBufferQueueItf bufferQueue = nullptr;
 
         if ((*this->d->m_audioRecorder)->GetInterface(this->d->m_audioRecorder,
                                                       SL_IID_ANDROIDSIMPLEBUFFERQUEUE,
-                                                      &bufferQueue) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                                      &bufferQueue) != SL_RESULT_SUCCESS) {
+            (*this->d->m_audioRecorder)->Destroy(this->d->m_audioRecorder);
+            this->d->m_audioRecorder = nullptr;
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         if ((*bufferQueue)->RegisterCallback(bufferQueue,
                                              AudioDevOpenSLPrivate::sampleProcess,
-                                             this->d) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                             this->d) != SL_RESULT_SUCCESS) {
+            (*this->d->m_audioRecorder)->Destroy(this->d->m_audioRecorder);
+            this->d->m_audioRecorder = nullptr;
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         auto bufferSize = this->d->self->latency()
                           * this->d->m_curCaps.bps()
@@ -176,72 +242,133 @@ bool AudioDevOpenSL::init(const QString &device, const AkAudioCaps &caps)
         AudioDevOpenSLPrivate::sampleProcess(bufferQueue, this->d);
 
         if ((*audioRecorder)->SetPlayState(audioRecorder,
-                                           SL_RECORDSTATE_RECORDING) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                           SL_RECORDSTATE_RECORDING) != SL_RESULT_SUCCESS) {
+            (*this->d->m_audioRecorder)->Destroy(this->d->m_audioRecorder);
+            this->d->m_audioRecorder = nullptr;
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
     } else {
         this->d->m_outputMix =
                 AudioDevOpenSLPrivate::createOutputMix(this->d->m_engine);
 
-        if (!this->d->m_outputMix)
-            goto init_failed;
+        if (!this->d->m_outputMix) {
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         this->d->m_audioPlayer =
                 AudioDevOpenSLPrivate::createAudioPlayer(this->d->m_engine,
                                                          this->d->m_outputMix,
                                                          caps);
 
-        if (!this->d->m_audioPlayer)
-            goto init_failed;
+        if (!this->d->m_audioPlayer) {
+            if (this->d->m_outputMix) {
+                (*this->d->m_outputMix)->Destroy(this->d->m_outputMix);
+                this->d->m_outputMix = nullptr;
+            }
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         SLPlayItf audioPlayer = nullptr;
 
         if ((*this->d->m_audioPlayer)->GetInterface(this->d->m_audioPlayer,
                                                     SL_IID_PLAY,
-                                                    &audioPlayer) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                                    &audioPlayer) != SL_RESULT_SUCCESS) {
+            if (this->d->m_audioPlayer) {
+                (*this->d->m_audioPlayer)->Destroy(this->d->m_audioPlayer);
+                this->d->m_audioPlayer = nullptr;
+            }
+
+            if (this->d->m_outputMix) {
+                (*this->d->m_outputMix)->Destroy(this->d->m_outputMix);
+                this->d->m_outputMix = nullptr;
+            }
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         SLAndroidSimpleBufferQueueItf bufferQueue = nullptr;
 
         if ((*this->d->m_audioPlayer)->GetInterface(this->d->m_audioPlayer,
                                                     SL_IID_ANDROIDSIMPLEBUFFERQUEUE,
-                                                    &bufferQueue) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                                    &bufferQueue) != SL_RESULT_SUCCESS) {
+            if (this->d->m_audioPlayer) {
+                (*this->d->m_audioPlayer)->Destroy(this->d->m_audioPlayer);
+                this->d->m_audioPlayer = nullptr;
+            }
+
+            if (this->d->m_outputMix) {
+                (*this->d->m_outputMix)->Destroy(this->d->m_outputMix);
+                this->d->m_outputMix = nullptr;
+            }
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         if ((*bufferQueue)->RegisterCallback(bufferQueue,
                                              AudioDevOpenSLPrivate::sampleProcess,
-                                             this->d) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                             this->d) != SL_RESULT_SUCCESS) {
+            if (this->d->m_audioPlayer) {
+                (*this->d->m_audioPlayer)->Destroy(this->d->m_audioPlayer);
+                this->d->m_audioPlayer = nullptr;
+            }
+
+            if (this->d->m_outputMix) {
+                (*this->d->m_outputMix)->Destroy(this->d->m_outputMix);
+                this->d->m_outputMix = nullptr;
+            }
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         if ((*audioPlayer)->SetPlayState(audioPlayer,
-                                         SL_PLAYSTATE_PLAYING) != SL_RESULT_SUCCESS)
-            goto init_failed;
+                                         SL_PLAYSTATE_PLAYING) != SL_RESULT_SUCCESS) {
+            if (this->d->m_audioPlayer) {
+                (*this->d->m_audioPlayer)->Destroy(this->d->m_audioPlayer);
+                this->d->m_audioPlayer = nullptr;
+            }
+
+            if (this->d->m_outputMix) {
+                (*this->d->m_outputMix)->Destroy(this->d->m_outputMix);
+                this->d->m_outputMix = nullptr;
+            }
+
+            (*this->d->m_engine)->Destroy(this->d->m_engine);
+            this->d->m_engine = nullptr;
+            this->d->m_audioBuffers.clear();
+
+            return false;
+        }
 
         AudioDevOpenSLPrivate::sampleProcess(bufferQueue, this->d);
     }
 
     return true;
-
-init_failed:
-    if (this->d->m_audioRecorder) {
-        (*this->d->m_audioRecorder)->Destroy(this->d->m_audioRecorder);
-        this->d->m_audioRecorder = nullptr;
-    }
-
-    if (this->d->m_audioPlayer) {
-        (*this->d->m_audioPlayer)->Destroy(this->d->m_audioPlayer);
-        this->d->m_audioPlayer = nullptr;
-    }
-
-    if (this->d->m_outputMix) {
-        (*this->d->m_outputMix)->Destroy(this->d->m_outputMix);
-        this->d->m_outputMix = nullptr;
-    }
-
-    (*this->d->m_engine)->Destroy(this->d->m_engine);
-    this->d->m_engine = nullptr;
-    this->d->m_audioBuffers.clear();
-
-    return false;
 }
 
 QByteArray AudioDevOpenSL::read()
@@ -264,7 +391,7 @@ bool AudioDevOpenSL::write(const AkAudioPacket &packet)
     if (this->d->m_audioBuffers.size() >= N_BUFFERS)
         this->d->m_bufferReady.wait(&this->d->m_mutex);
 
-    this->d->m_audioBuffers << packet.buffer();
+    this->d->m_audioBuffers << QByteArray(packet.constData(), packet.size());
     this->d->m_mutex.unlock();
 
     return true;
@@ -321,18 +448,26 @@ AudioDevOpenSLPrivate::AudioDevOpenSLPrivate(AudioDevOpenSL *self):
 SLObjectItf AudioDevOpenSLPrivate::createEngine()
 {
     SLObjectItf engineObject = nullptr;
+    auto result = slCreateEngine(&engineObject,
+                                 0,
+                                 nullptr,
+                                 0,
+                                 nullptr,
+                                 nullptr);
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Failed to create OpenSL engine:"
+                << slErrorToStrMap->value(result);
 
-    if (slCreateEngine(&engineObject,
-                       0,
-                       nullptr,
-                       0,
-                       nullptr,
-                       nullptr) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
-    if ((*engineObject)->Realize(engineObject,
-                                 SL_BOOLEAN_FALSE) != SL_RESULT_SUCCESS) {
+    result = (*engineObject)->Realize(engineObject, SL_BOOLEAN_FALSE);
+
+    if (result != SL_RESULT_SUCCESS) {
         (*engineObject)->Destroy(engineObject);
+        qInfo() << "Can't realize OpenSL engine:"
+                << slErrorToStrMap->value(result);
 
         return nullptr;
     }
@@ -343,24 +478,37 @@ SLObjectItf AudioDevOpenSLPrivate::createEngine()
 SLObjectItf AudioDevOpenSLPrivate::createOutputMix(SLObjectItf engine)
 {
     SLEngineItf engineInterface = nullptr;
+    auto result = (*engine)->GetInterface(engine,
+                                          SL_IID_ENGINE,
+                                          &engineInterface);
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Can't get engine interface:"
+                << slErrorToStrMap->value(result);
 
-    if ((*engine)->GetInterface(engine,
-                                SL_IID_ENGINE,
-                                &engineInterface) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
     SLObjectItf outputMixObject = nullptr;
+    result = (*engineInterface)->CreateOutputMix(engineInterface,
+                                                 &outputMixObject,
+                                                 0,
+                                                 nullptr,
+                                                 nullptr);
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Can't create OutputMix:"
+                << slErrorToStrMap->value(result);
 
-    if ((*engineInterface)->CreateOutputMix(engineInterface,
-                                            &outputMixObject,
-                                            0,
-                                            nullptr,
-                                            nullptr) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
-    if ((*outputMixObject)->Realize(outputMixObject,
-                                    SL_BOOLEAN_FALSE) != SL_RESULT_SUCCESS) {
+    result = (*outputMixObject)->Realize(outputMixObject, SL_BOOLEAN_FALSE);
+
+    if (result != SL_RESULT_SUCCESS) {
         (*outputMixObject)->Destroy(outputMixObject);
+        qInfo() << "Can't realize OutputMix:"
+                << slErrorToStrMap->value(result);
 
         return nullptr;
     }
@@ -376,11 +524,16 @@ SLObjectItf AudioDevOpenSLPrivate::createAudioPlayer(SLObjectItf engine,
         return nullptr;
 
     SLEngineItf engineInterface = nullptr;
+    auto result = (*engine)->GetInterface(engine,
+                                          SL_IID_ENGINE,
+                                          &engineInterface);
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Can't get engine interface:"
+                << slErrorToStrMap->value(result);
 
-    if ((*engine)->GetInterface(engine,
-                                SL_IID_ENGINE,
-                                &engineInterface) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
     SLObjectItf audioPlayerObject = nullptr;
 
@@ -407,19 +560,27 @@ SLObjectItf AudioDevOpenSLPrivate::createAudioPlayer(SLObjectItf engine,
     };
     auto requiredKeys = interfaces.keys().toVector();
     auto requiredValues = interfaces.values().toVector();
+    result = (*engineInterface)->CreateAudioPlayer(engineInterface,
+                                                   &audioPlayerObject,
+                                                   &dataSource,
+                                                   &dataSink,
+                                                   SLuint32(interfaces.size()),
+                                                   requiredKeys.data(),
+                                                   requiredValues.data());
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Can't create AudioPlayer:"
+                << slErrorToStrMap->value(result);
 
-    if ((*engineInterface)->CreateAudioPlayer(engineInterface,
-                                              &audioPlayerObject,
-                                              &dataSource,
-                                              &dataSink,
-                                              SLuint32(interfaces.size()),
-                                              requiredKeys.data(),
-                                              requiredValues.data()) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
-    if ((*audioPlayerObject)->Realize(audioPlayerObject,
-                                      SL_BOOLEAN_FALSE) != SL_RESULT_SUCCESS) {
+    result = (*audioPlayerObject)->Realize(audioPlayerObject, SL_BOOLEAN_FALSE);
+
+    if (result != SL_RESULT_SUCCESS) {
         (*audioPlayerObject)->Destroy(audioPlayerObject);
+        qInfo() << "Can't realize AudioPlayer:"
+                << slErrorToStrMap->value(result);
 
         return nullptr;
     }
@@ -431,11 +592,16 @@ SLObjectItf AudioDevOpenSLPrivate::createAudioRecorder(SLObjectItf engine,
                                                        const AkAudioCaps &caps)
 {
     SLEngineItf engineInterface = nullptr;
+    auto result = (*engine)->GetInterface(engine,
+                                          SL_IID_ENGINE,
+                                          &engineInterface);
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Can't get engine interface:"
+                << slErrorToStrMap->value(result);
 
-    if ((*engine)->GetInterface(engine,
-                                SL_IID_ENGINE,
-                                &engineInterface) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
     SLObjectItf audioRecorderObject = nullptr;
     SLDataLocator_IODevice ioDeviceLocator {
@@ -463,19 +629,28 @@ SLObjectItf AudioDevOpenSLPrivate::createAudioRecorder(SLObjectItf engine,
     };
     auto requiredKeys = interfaces.keys().toVector();
     auto requiredValues = interfaces.values().toVector();
+    result = (*engineInterface)->CreateAudioRecorder(engineInterface,
+                                                     &audioRecorderObject,
+                                                     &dataSource,
+                                                     &dataSink,
+                                                     SLuint32(interfaces.size()),
+                                                     requiredKeys.data(),
+                                                     requiredValues.data());
+
+    if (result != SL_RESULT_SUCCESS) {
+        qInfo() << "Can't create AudioRecorder:"
+                << slErrorToStrMap->value(result);
 
-    if ((*engineInterface)->CreateAudioRecorder(engineInterface,
-                                                &audioRecorderObject,
-                                                &dataSource,
-                                                &dataSink,
-                                                SLuint32(interfaces.size()),
-                                                requiredKeys.data(),
-                                                requiredValues.data()) != SL_RESULT_SUCCESS)
         return nullptr;
+    }
 
-    if ((*audioRecorderObject)->Realize(audioRecorderObject,
-                                        SL_BOOLEAN_FALSE) != SL_RESULT_SUCCESS) {
+    result = (*audioRecorderObject)->Realize(audioRecorderObject,
+                                             SL_BOOLEAN_FALSE);
+
+    if (result != SL_RESULT_SUCCESS) {
         (*audioRecorderObject)->Destroy(audioRecorderObject);
+        qInfo() << "Can't realize AudioRecorder:"
+                << slErrorToStrMap->value(result);
 
         return nullptr;
     }
@@ -614,6 +789,42 @@ void AudioDevOpenSLPrivate::sampleProcess(SLAndroidSimpleBufferQueueItf bufferQu
     }
 }
 
+bool AudioDevOpenSLPrivate::hasAudioCapturePermissions()
+{
+#ifdef Q_OS_ANDROID
+    static bool done = false;
+    static bool result = false;
+
+    if (done)
+        return result;
+
+    QStringList permissions {
+        "android.permission.RECORD_AUDIO"
+    };
+    QStringList neededPermissions;
+
+    for (auto &permission: permissions)
+        if (QtAndroid::checkPermission(permission) == QtAndroid::PermissionResult::Denied)
+            neededPermissions << permission;
+
+    if (!neededPermissions.isEmpty()) {
+        auto results = QtAndroid::requestPermissionsSync(neededPermissions);
+
+        for (auto it = results.constBegin(); it != results.constEnd(); it++)
+            if (it.value() == QtAndroid::PermissionResult::Denied) {
+                done = true;
+
+                return false;
+            }
+    }
+
+    done = true;
+    result = true;
+#endif
+
+    return true;
+}
+
 void AudioDevOpenSLPrivate::updateDevices()
 {
     auto engine = AudioDevOpenSLPrivate::createEngine();
@@ -653,38 +864,41 @@ void AudioDevOpenSLPrivate::updateDevices()
         SL_SAMPLINGRATE_192	   / 1000,
     };
 
-    // Test audio input
-    for (auto &format: sampleFormats)
-        for (auto &layout: layouts)
-            for (auto &rate: sampleRates) {
-                AkAudioCaps caps(format, layout, rate);
-                auto audioRecorder =
-                        AudioDevOpenSLPrivate::createAudioRecorder(engine, caps);
+    if (this->hasAudioCapturePermissions()) {
+        // Test audio input
+        for (auto &format: sampleFormats)
+            for (auto &layout: layouts)
+                for (auto &rate: sampleRates) {
+                    AkAudioCaps caps(format, layout, false, rate);
+                    auto audioRecorder =
+                            AudioDevOpenSLPrivate::createAudioRecorder(engine, caps);
 
-                if (audioRecorder) {
-                    if (!this->m_supportedFormats[":openslinput:"].contains(format))
-                        this->m_supportedFormats[":openslinput:"] << format;
+                    if (audioRecorder) {
+                        if (!this->m_supportedFormats[":openslinput:"].contains(format))
+                            this->m_supportedFormats[":openslinput:"] << format;
 
-                    if (!this->m_supportedLayouts[":openslinput:"].contains(layout))
-                        this->m_supportedLayouts[":openslinput:"] << layout;
+                        if (!this->m_supportedLayouts[":openslinput:"].contains(layout))
+                            this->m_supportedLayouts[":openslinput:"] << layout;
 
-                    if (!this->m_supportedSampleRates[":openslinput:"].contains(rate))
-                        this->m_supportedSampleRates[":openslinput:"] << rate;
+                        if (!this->m_supportedSampleRates[":openslinput:"].contains(rate))
+                            this->m_supportedSampleRates[":openslinput:"] << rate;
 
-                    (*audioRecorder)->Destroy(audioRecorder);
+                        (*audioRecorder)->Destroy(audioRecorder);
+                    }
                 }
-            }
 
-    if (this->m_supportedFormats.contains(":openslinput:")
-        && this->m_supportedLayouts.contains(":openslinput:")
-        && this->m_supportedSampleRates.contains(":openslinput:")) {
-        this->m_sources = QStringList {":openslinput:"};
-        this->m_pinDescriptionMap[":openslinput:"] = "OpenSL ES Input";
-        this->m_preferredCaps[":openslinput:"] = {
-            AkAudioCaps::SampleFormat_s16,
-            AkAudioCaps::Layout_mono,
-            44100,
-        };
+        if (this->m_supportedFormats.contains(":openslinput:")
+            && this->m_supportedLayouts.contains(":openslinput:")
+            && this->m_supportedSampleRates.contains(":openslinput:")) {
+            this->m_sources = QStringList {":openslinput:"};
+            this->m_pinDescriptionMap[":openslinput:"] = "OpenSL ES Input";
+            this->m_preferredCaps[":openslinput:"] = {
+                AkAudioCaps::SampleFormat_s16,
+                AkAudioCaps::Layout_mono,
+                false,
+                44100,
+            };
+        }
     }
 
     // Test audio output
@@ -694,7 +908,7 @@ void AudioDevOpenSLPrivate::updateDevices()
         for (auto &format: sampleFormats)
             for (auto &layout: layouts)
                 for (auto &rate: sampleRates) {
-                    AkAudioCaps caps(format, layout, rate);
+                    AkAudioCaps caps(format, layout, false, rate);
                     auto audioPlayer =
                             AudioDevOpenSLPrivate::createAudioPlayer(engine,
                                                                      outputMix,
@@ -722,6 +936,7 @@ void AudioDevOpenSLPrivate::updateDevices()
             this->m_preferredCaps[":opensloutput:"] = {
                 AkAudioCaps::SampleFormat_s16,
                 AkAudioCaps::Layout_stereo,
+                false,
                 44100,
             };
         }
