@@ -22,17 +22,15 @@
 
 QObject *ChangeHSL::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new ChangeHSLElement();
-
-    return nullptr;
+    return new ChangeHSLElement();
 }
 
 QStringList ChangeHSL::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_changehsl.cpp"
