@@ -22,17 +22,15 @@
 
 QObject *ColorTransform::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new ColorTransformElement();
-
-    return nullptr;
+    return new ColorTransformElement();
 }
 
 QStringList ColorTransform::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_colortransform.cpp"
