@@ -17,6 +17,8 @@
  * Web-Site: http://webcamoid.github.io/
  */
 
+#include <QQmlEngine>
+
 #include "delaygrab.h"
 #include "delaygrabelement.h"
 
@@ -24,6 +26,9 @@ QObject *DelayGrab::create(const QString &key, const QString &specification)
 {
     Q_UNUSED(key)
     Q_UNUSED(specification)
+    qRegisterMetaType<DelayGrabElement::DelayGrabMode>("DelayGrabDelayGrabMode");
+    qRegisterMetaTypeStreamOperators<DelayGrabElement::DelayGrabMode>("DelayGrabDelayGrabMode");
+    qmlRegisterType<DelayGrabElement>("DelayGrabElement", 1, 0, "DelayGrabElement");
 
     return new DelayGrabElement();
 }
