@@ -20,6 +20,7 @@
 #ifndef DESKTOPCAPTUREELEMENT_H
 #define DESKTOPCAPTUREELEMENT_H
 
+#include "akvideocaps.h"
 #include <akmultimediasourceelement.h>
 
 class DesktopCaptureElementPrivate;
@@ -57,20 +58,20 @@ class DesktopCaptureElement: public AkMultimediaSourceElement
         ~DesktopCaptureElement();
 
         Q_INVOKABLE AkFrac fps() const;
-        Q_INVOKABLE QStringList medias();
-        Q_INVOKABLE QString media() const;
-        Q_INVOKABLE QList<int> streams();
-        Q_INVOKABLE int defaultStream(const QString &mimeType);
-        Q_INVOKABLE QString description(const QString &media);
-        Q_INVOKABLE AkCaps caps(int stream);
+        Q_INVOKABLE QStringList medias() override;
+        Q_INVOKABLE QString media() const override;
+        Q_INVOKABLE QList<int> streams() override;
+        Q_INVOKABLE int defaultStream(AkCaps::CapsType type) override;
+        Q_INVOKABLE QString description(const QString &media) override;
+        Q_INVOKABLE AkCaps caps(int stream) override;
 
     private:
         DesktopCaptureElementPrivate *d;
 
     protected:
-        QString controlInterfaceProvide(const QString &controlId) const;
+        QString controlInterfaceProvide(const QString &controlId) const override;
         void controlInterfaceConfigure(QQmlContext *context,
-                                       const QString &controlId) const;
+                                       const QString &controlId) const override;
 
     signals:
         void mediasChanged(const QStringList &medias);
@@ -84,9 +85,9 @@ class DesktopCaptureElement: public AkMultimediaSourceElement
     public slots:
         void setFps(const AkFrac &fps);
         void resetFps();
-        void setMedia(const QString &media);
-        void resetMedia();
-        bool setState(AkElement::ElementState state);
+        void setMedia(const QString &media) override;
+        void resetMedia() override;
+        bool setState(AkElement::ElementState state) override;
 };
 
 #endif // DESKTOPCAPTUREELEMENT_H
