@@ -22,17 +22,15 @@
 
 QObject *Equalize::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new EqualizeElement();
-
-    return nullptr;
+    return new EqualizeElement();
 }
 
 QStringList Equalize::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_equalize.cpp"
