@@ -22,17 +22,15 @@
 
 QObject *GrayScale::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new GrayScaleElement();
-
-    return nullptr;
+    return new GrayScaleElement();
 }
 
 QStringList GrayScale::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_grayscale.cpp"
