@@ -22,17 +22,15 @@
 
 QObject *Halftone::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new HalftoneElement();
-
-    return nullptr;
+    return new HalftoneElement();
 }
 
 QStringList Halftone::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_halftone.cpp"
