@@ -17,22 +17,25 @@
  * Web-Site: http://webcamoid.github.io/
  */
 
+#include <QQmlEngine>
+
 #include "hypnotic.h"
 #include "hypnoticelement.h"
 
 QObject *Hypnotic::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
+    qRegisterMetaType<HypnoticElement::OpticMode>("HypnoticOpticMode");
+    qRegisterMetaTypeStreamOperators<HypnoticElement::OpticMode>("HypnoticOpticMode");
+    qmlRegisterType<HypnoticElement>("HypnoticElement", 1, 0, "HypnoticElement");
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new HypnoticElement();
-
-    return nullptr;
+    return new HypnoticElement();
 }
 
 QStringList Hypnotic::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_hypnotic.cpp"
