@@ -22,17 +22,15 @@
 
 QObject *Nervous::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new NervousElement();
-
-    return nullptr;
+    return new NervousElement();
 }
 
 QStringList Nervous::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_nervous.cpp"
