@@ -22,17 +22,15 @@
 
 QObject *Normalize::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new NormalizeElement();
-
-    return nullptr;
+    return new NormalizeElement();
 }
 
 QStringList Normalize::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_normalize.cpp"
