@@ -22,17 +22,15 @@
 
 QObject *Shagadelic::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new ShagadelicElement();
-
-    return nullptr;
+    return new ShagadelicElement();
 }
 
 QStringList Shagadelic::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_shagadelic.cpp"
