@@ -22,17 +22,15 @@
 
 QObject *Temperature::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new TemperatureElement();
-
-    return nullptr;
+    return new TemperatureElement();
 }
 
 QStringList Temperature::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_temperature.cpp"
