@@ -51,24 +51,43 @@ class VideoCaptureElement: public AkMultimediaSourceElement
     Q_PROPERTY(QString ioMethod
                READ ioMethod
                WRITE setIoMethod
-               RESET resetIoMethod)
+               RESET resetIoMethod
+               NOTIFY ioMethodChanged)
     Q_PROPERTY(int nBuffers
                READ nBuffers
                WRITE setNBuffers
-               RESET resetNBuffers)
+               RESET resetNBuffers
+               NOTIFY nBuffersChanged)
+    Q_PROPERTY(FlashMode flashMode
+               READ flashMode
+               WRITE setFlashMode
+               RESET resetFlashMode
+               NOTIFY flashModeChanged)
 
     public:
+        enum FlashMode
+        {
+            FlashMode_Off,
+            FlashMode_On,
+            FlashMode_Auto,
+            FlashMode_Torch,
+            FlashMode_RedEye,
+            FlashMode_External,
+        };
+        Q_ENUM(FlashMode)
+        using FlashModeList = QList<FlashMode>;
+
         VideoCaptureElement();
         ~VideoCaptureElement();
 
         Q_INVOKABLE QString error() const;
-        Q_INVOKABLE QStringList medias();
-        Q_INVOKABLE QString media() const;
-        Q_INVOKABLE QList<int> streams();
-        Q_INVOKABLE QList<int> listTracks(const QString &mimeType={});
-        Q_INVOKABLE int defaultStream(const QString &mimeType);
-        Q_INVOKABLE QString description(const QString &media);
-        Q_INVOKABLE AkCaps caps(int stream);
+        Q_INVOKABLE QStringList medias() override;
+        Q_INVOKABLE QString media() const override;
+        Q_INVOKABLE QList<int> streams() override;
+        Q_INVOKABLE QList<int> listTracks(AkCaps::CapsType type=AkCaps::CapsUnknown);
+        Q_INVOKABLE int defaultStream(AkCaps::CapsType type) override;
+        Q_INVOKABLE QString description(const QString &media) override;
+        Q_INVOKABLE AkCaps caps(int stream) override;
         Q_INVOKABLE AkCaps rawCaps(int stream) const;
         Q_INVOKABLE QString streamDescription(int stream) const;
         Q_INVOKABLE QStringList listCapsDescription() const;
@@ -80,14 +99,16 @@ class VideoCaptureElement: public AkMultimediaSourceElement
         Q_INVOKABLE QVariantList cameraControls() const;
         Q_INVOKABLE bool setCameraControls(const QVariantMap &cameraControls);
         Q_INVOKABLE bool resetCameraControls();
+        Q_INVOKABLE FlashModeList supportedFlashModes(const QString &webcam) const;
+        Q_INVOKABLE FlashMode flashMode() const;
 
     private:
         VideoCaptureElementPrivate *d;
 
     protected:
-        QString controlInterfaceProvide(const QString &controlId) const;
+        QString controlInterfaceProvide(const QString &controlId) const override;
         void controlInterfaceConfigure(QQmlContext *context,
-                                       const QString &controlId) const;
+                                       const QString &controlId) const override;
 
     signals:
         void errorChanged(const QString &error);
@@ -95,20 +116,30 @@ class VideoCaptureElement: public AkMultimediaSourceElement
         void mediaChanged(const QString &media);
         void streamsChanged(const QList<int> &streams);
         void loopChanged(bool loop);
-        void imageControlsChanged(const QVariantMap &imageControls) const;
-        void cameraControlsChanged(const QVariantMap &cameraControls) const;
+        void ioMethodChanged(const QString &ioMethod);
+        void nBuffersChanged(int nBuffers);
+        void imageControlsChanged(const QVariantMap &imageControls);
+        void cameraControlsChanged(const QVariantMap &cameraControls);
+        void pictureTaken(int index, const AkPacket &picture);
+        void flashModeChanged(FlashMode mode);
 
     public slots:
-        void setMedia(const QString &media);
-        void setStreams(const QList<int> &streams);
+        void setMedia(const QString &media) override;
+        void setStreams(const QList<int> &streams) override;
         void setIoMethod(const QString &ioMethod);
         void setNBuffers(int nBuffers);
-        void resetMedia();
-        void resetStreams();
+        void setFlashMode(FlashMode mode);
+        void resetMedia() override;
+        void resetStreams() override;
         void resetIoMethod();
         void resetNBuffers();
+        void resetFlashMode();
         void reset();
-        bool setState(AkElement::ElementState state);
+        void takePictures(int count, int delayMsecs=0);
+        bool setState(AkElement::ElementState state) override;
 };
 
+Q_DECLARE_METATYPE(VideoCaptureElement::FlashMode)
+Q_DECLARE_METATYPE(VideoCaptureElement::FlashModeList)
+
 #endif // VIDEOCAPTUREELEMENT_H
