@@ -34,6 +34,7 @@
 #include <QWaitCondition>
 #include <QXmlStreamReader>
 #include <akfrac.h>
+#include <akvideoconverter.h>
 
 #include "vcamcmio.h"
 
@@ -84,8 +85,11 @@ class VCamCMIOPrivate
         QMutex m_controlsMutex;
         QString m_error;
         AkVideoCaps m_currentCaps;
+        AkVideoConverter m_videoConverter;
         QString m_picture;
         QString m_rootMethod;
+        bool m_isInitialized {false};
+        bool m_runEventsProc {true};
 
         VCamCMIOPrivate(VCamCMIO *self=nullptr);
         ~VCamCMIOPrivate();
@@ -130,6 +134,7 @@ VCamCMIO::VCamCMIO(QObject *parent):
 
 VCamCMIO::~VCamCMIO()
 {
+    this->uninit();
     delete this->d;
 }
 
@@ -679,11 +684,19 @@ bool VCamCMIO::destroyAllDevices()
 
 bool VCamCMIO::init()
 {
+    this->d->m_isInitialized = false;
+
+    if (this->d->m_device.isEmpty() || this->d->m_devices.isEmpty())
+        return false;
+
     auto manager = this->d->manager();
 
     if (manager.isEmpty())
         return false;
 
+    auto outputCaps = this->d->m_currentCaps;
+    outputCaps.setFormat(AkVideoCaps::Format_rgb24);
+
     QString params;
     QTextStream paramsStream(&params);
     paramsStream << "\""
@@ -693,16 +706,19 @@ bool VCamCMIO::init()
                  << " "
                  << this->d->m_device
                  << " "
-                 << this->d->cmioAkFormatMap().value(this->d->m_currentCaps.format())
+                 << this->d->cmioAkFormatMap().value(outputCaps.format())
                  << " "
-                 << this->d->m_currentCaps.width()
+                 << outputCaps.width()
                  << " "
-                 << this->d->m_currentCaps.height();
+                 << outputCaps.height();
     this->d->m_streamProc = popen(params.toStdString().c_str(), "w");
 
     if (this->d->m_streamProc)
         this->d->m_curFormat = this->d->m_currentCaps;
 
+    this->d->m_videoConverter.setOutputCaps(outputCaps);
+    this->d->m_isInitialized = true;
+
     return this->d->m_streamProc != nullptr;
 }
 
@@ -713,7 +729,7 @@ void VCamCMIO::uninit()
         this->d->m_streamProc = nullptr;
     }
 
-    this->d->m_curFormat.clear();
+    this->d->m_curFormat = AkVideoCaps();
 }
 
 void VCamCMIO::setDevice(const QString &device)
@@ -824,6 +840,9 @@ bool VCamCMIO::applyPicture()
 
 bool VCamCMIO::write(const AkVideoPacket &frame)
 {
+    if (!this->d->m_isInitialized)
+        return false;
+
     if (!this->d->m_streamProc)
         return false;
 
@@ -838,17 +857,25 @@ bool VCamCMIO::write(const AkVideoPacket &frame)
         this->d->m_localControls = curControls;
     }
 
-    auto scaled = frame.scaled(this->d->m_curFormat.width(),
-                               this->d->m_curFormat.height())
-                        .convert(this->d->m_curFormat.format());
+    this->d->m_videoConverter.begin();
+    auto videoPacket = this->d->m_videoConverter.convert(frame);
+    this->d->m_videoConverter.end();
 
-    if (!scaled)
+    if (!videoPacket)
         return false;
 
-    return fwrite(scaled.buffer().data(),
-                  size_t(scaled.buffer().size()),
-                  1,
-                  this->d->m_streamProc) > 0;
+    bool ok = true;
+
+    for (int y = 0; y < videoPacket.caps().height(); y++) {
+        auto line = videoPacket.constLine(0, y);
+        auto lineSize = videoPacket.bytesUsed(0);
+        ok = fwrite(line, lineSize, 1, this->d->m_streamProc) > 0;
+
+        if (!ok)
+            break;
+    }
+
+    return ok;
 }
 
 VCamCMIOPrivate::VCamCMIOPrivate(VCamCMIO *self):
@@ -864,7 +891,7 @@ VCamCMIOPrivate::VCamCMIOPrivate(VCamCMIO *self):
         QObject::connect(this->m_eventsProc,
                          &QProcess::readyReadStandardOutput,
                          [this] () {
-            while (this->m_eventsProc->canReadLine()) {
+            while (this->m_runEventsProc && this->m_eventsProc->canReadLine()) {
                 auto event = this->m_eventsProc->readLine().trimmed();
                 qDebug() << "Event:" << event;
 
@@ -885,7 +912,9 @@ VCamCMIOPrivate::VCamCMIOPrivate(VCamCMIO *self):
 VCamCMIOPrivate::~VCamCMIOPrivate()
 {
     if (this->m_eventsProc) {
-        this->m_eventsProc->terminate();
+        this->m_runEventsProc = false;
+        //this->m_eventsProc->terminate();
+        this->m_eventsProc->kill();
         this->m_eventsProc->waitForFinished();
         delete this->m_eventsProc;
     }
@@ -1051,12 +1080,14 @@ QVariantList VCamCMIOPrivate::controls(const QString &device)
         case QXmlStreamReader::StartElement: {
             pathList << xmlInfo.name().toString();
 
-            if (path.isEmpty() && xmlInfo.name() != "info")
+            if (path.isEmpty() && xmlInfo.name() != QStringLiteral("info"))
                 return {};
 
-            if (path == "info/devices" && xmlInfo.name() == "device")
+            if (path == "info/devices"
+                && xmlInfo.name() == QStringLiteral("device"))
                 curDeviceControls.clear();
-            else if (path == "info/devices/device/controls" && xmlInfo.name() == "control")
+            else if (path == "info/devices/device/controls"
+                     && xmlInfo.name() == QStringLiteral("control"))
                 deviceControl = {};
 
             break;
@@ -1245,12 +1276,14 @@ void VCamCMIOPrivate::updateDevices()
         case QXmlStreamReader::StartElement: {
             pathList << xmlInfo.name().toString();
 
-            if (path.isEmpty() && xmlInfo.name() != "info")
+            if (path.isEmpty() && xmlInfo.name() != QStringLiteral("info"))
                 return;
 
-            if (path == "info/devices" && xmlInfo.name() == "device")
+            if (path == "info/devices"
+                && xmlInfo.name() == QStringLiteral("device"))
                 curDeviceCaps.clear();
-            else if (path == "info/devices/device/formats" && xmlInfo.name() == "format")
+            else if (path == "info/devices/device/formats"
+                     && xmlInfo.name() == QStringLiteral("format"))
                 curFormat = {};
 
             break;
