@@ -35,6 +35,7 @@
 #include <QXmlStreamReader>
 #include <windows.h>
 #include <akfrac.h>
+#include <akvideoconverter.h>
 
 #include "vcamdshow.h"
 
@@ -95,8 +96,11 @@ class VCamDShowPrivate
         QMutex m_controlsMutex;
         QString m_error;
         AkVideoCaps m_currentCaps;
+        AkVideoConverter m_videoConverter;
         QString m_picture;
         QString m_rootMethod;
+        bool m_isInitialized {false};
+        bool m_runEventsProc {true};
 
         VCamDShowPrivate(VCamDShow *self=nullptr);
         ~VCamDShowPrivate();
@@ -147,6 +151,7 @@ VCamDShow::VCamDShow(QObject *parent):
 
 VCamDShow::~VCamDShow()
 {
+    this->uninit();
     delete this->d;
 }
 
@@ -886,11 +891,19 @@ bool VCamDShow::destroyAllDevices()
 
 bool VCamDShow::init()
 {
+    this->d->m_isInitialized = false;
+
+    if (this->d->m_device.isEmpty() || this->d->m_devices.isEmpty())
+        return false;
+
     auto manager = this->d->manager();
 
     if (manager.isEmpty())
         return false;
 
+    auto outputCaps = this->d->m_currentCaps;
+    outputCaps.setFormat(AkVideoCaps::Format_rgb24);
+
     QString params;
     QTextStream paramsStream(&params);
     paramsStream << "\""
@@ -900,11 +913,11 @@ bool VCamDShow::init()
                  << " "
                  << this->d->m_device
                  << " "
-                 << this->d->dshowAkFormatMap().value(this->d->m_currentCaps.format())
+                 << this->d->dshowAkFormatMap().value(outputCaps.format())
                  << " "
-                 << this->d->m_currentCaps.width()
+                 << outputCaps.width()
                  << " "
-                 << this->d->m_currentCaps.height();
+                 << outputCaps.height();
 
     this->d->m_streamProc.stdinReadPipe = nullptr;
     this->d->m_streamProc.stdinWritePipe = nullptr;
@@ -957,7 +970,8 @@ bool VCamDShow::init()
         return false;
     }
 
-    this->d->m_curFormat = this->d->m_currentCaps;
+    this->d->m_videoConverter.setOutputCaps(outputCaps);
+    this->d->m_isInitialized = true;
 
     return true;
 }
@@ -976,8 +990,6 @@ void VCamDShow::uninit()
         CloseHandle(this->d->m_streamProc.procInfo.hProcess);
         CloseHandle(this->d->m_streamProc.procInfo.hThread);
     }
-
-    this->d->m_curFormat.clear();
 }
 
 void VCamDShow::setDevice(const QString &device)
@@ -1145,6 +1157,9 @@ bool VCamDShow::applyPicture()
 
 bool VCamDShow::write(const AkVideoPacket &frame)
 {
+    if (!this->d->m_isInitialized)
+        return false;
+
     if (!this->d->m_streamProc.stdinReadPipe)
         return false;
 
@@ -1159,11 +1174,11 @@ bool VCamDShow::write(const AkVideoPacket &frame)
         this->d->m_localControls = curControls;
     }
 
-    auto scaled = frame.scaled(this->d->m_curFormat.width(),
-                               this->d->m_curFormat.height())
-                        .convert(this->d->m_curFormat.format());
+    this->d->m_videoConverter.begin();
+    auto videoPacket = this->d->m_videoConverter.convert(frame);
+    this->d->m_videoConverter.end();
 
-    if (!scaled)
+    if (!videoPacket)
         return false;
 
     this->d->m_streamProc.stdinMutex.lock();
@@ -1172,9 +1187,9 @@ bool VCamDShow::write(const AkVideoPacket &frame)
     if (this->d->m_streamProc.stdinWritePipe) {
         ok = true;
 
-        for (int y = 0; y < scaled.caps().height(); y++) {
-            auto line = scaled.constLine(0, y);
-            auto lineSize = scaled.caps().bytesPerLine(0);
+        for (int y = 0; y < videoPacket.caps().height(); y++) {
+            auto line = videoPacket.constLine(0, y);
+            auto lineSize = videoPacket.bytesUsed(0);
             DWORD bytesWritten = 0;
             ok = WriteFile(this->d->m_streamProc.stdinWritePipe,
                            line,
@@ -1205,7 +1220,7 @@ VCamDShowPrivate::VCamDShowPrivate(VCamDShow *self):
         QObject::connect(this->m_eventsProc,
                          &QProcess::readyReadStandardOutput,
                          [this] () {
-            while (this->m_eventsProc->canReadLine()) {
+            while (this->m_runEventsProc && this->m_eventsProc->canReadLine()) {
                 auto event = this->m_eventsProc->readLine().trimmed();
                 qDebug() << "Event:" << event;
 
@@ -1226,7 +1241,9 @@ VCamDShowPrivate::VCamDShowPrivate(VCamDShow *self):
 VCamDShowPrivate::~VCamDShowPrivate()
 {
     if (this->m_eventsProc) {
-        this->m_eventsProc->terminate();
+        this->m_runEventsProc = false;
+        //this->m_eventsProc->terminate();
+        this->m_eventsProc->kill();
         this->m_eventsProc->waitForFinished();
         delete this->m_eventsProc;
     }
@@ -1393,12 +1410,14 @@ QVariantList VCamDShowPrivate::controls(const QString &device)
         case QXmlStreamReader::StartElement: {
             pathList << xmlInfo.name().toString();
 
-            if (path.isEmpty() && xmlInfo.name() != "info")
+            if (path.isEmpty() && xmlInfo.name() != QStringLiteral("info"))
                 return {};
 
-            if (path == "info/devices" && xmlInfo.name() == "device")
+            if (path == "info/devices"
+                && xmlInfo.name() == QStringLiteral("device"))
                 curDeviceControls.clear();
-            else if (path == "info/devices/device/controls" && xmlInfo.name() == "control")
+            else if (path == "info/devices/device/controls"
+                     && xmlInfo.name() == QStringLiteral("control"))
                 deviceControl = {};
 
             break;
@@ -1652,12 +1671,13 @@ void VCamDShowPrivate::updateDevices()
         case QXmlStreamReader::StartElement: {
             pathList << xmlInfo.name().toString();
 
-            if (path.isEmpty() && xmlInfo.name() != "info")
+            if (path.isEmpty() && xmlInfo.name() != QStringLiteral("info"))
                 return;
 
-            if (path == "info/devices" && xmlInfo.name() == "device")
+            if (path == "info/devices" && xmlInfo.name() == QStringLiteral("device"))
                 curDeviceCaps.clear();
-            else if (path == "info/devices/device/formats" && xmlInfo.name() == "format")
+            else if (path == "info/devices/device/formats"
+                     && xmlInfo.name() == QStringLiteral("format"))
                 curFormat = {};
 
             break;
