@@ -22,17 +22,15 @@
 
 QObject *Warhol::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new WarholElement();
-
-    return nullptr;
+    return new WarholElement();
 }
 
 QStringList Warhol::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_warhol.cpp"
